//	Bullet.cpp - Class representing a bullet.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "Bullet.h"
#include "Player.h"

//------------------------------------------------------------------------------
Bullet::Bullet(int dir, const shared_ptr<Player>& player):
position(player->getPosition()),
colour(player->getColour()),
parent(player)
{
	const float speed(6.0f);
	TwoFloats dirs[] = {TwoFloats(0.0f, -speed),
						TwoFloats(speed, 0.0f),
						TwoFloats(0.0f, speed),
						TwoFloats(-speed, 0.0f)};
	std::wstring ims[] = {L"Bullet0",
						  L"Bullet1",
						  L"Bullet2",
						  L"Bullet3"};

	direction = dirs[dir];
	image = ims[dir];
}

//------------------------------------------------------------------------------
void Bullet::update()
{
	position += direction;
}

//------------------------------------------------------------------------------
void Bullet::draw(shared_ptr<Drawer>& d)
{
	d->addImageToBatch(image,
					   position - 18.0f,
					   1.0f,
					   colour);
}
