//	Bullet.h - Class representing a bullet.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BULLET_H_
#define BULLET_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"

#include <string>

class Player;
class Drawer;

///	Class representing a bullet.
class Bullet
{
  public:
	///	Constructor
	Bullet(int dir, const shared_ptr<Player>& player);

	///	Moves the bullet.
	void update();
	///	Draws the bullet.
	void draw(shared_ptr<Drawer>& d);

	///	Returns the bullet's position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the bullet's direction.
	const TwoFloats& getDirection() const {return direction;};
	///	Returns the bullet's colour.
	const ThreeFloats& getColour() const {return colour;};
	///	Returns the bullet's parent Player.
	const shared_ptr<Player>& getParent() const {return parent;};
  private:
	///	The bullet's position.
	TwoFloats position;
	///	The bullet's direction.
	TwoFloats direction;
	///	The bullet's image.
	std::wstring image;
	///	The bullet's colour.
	ThreeFloats colour;	
	///	The bullet's parent.
	const shared_ptr<Player>& parent;
};

#endif
