//	ExplodeFlash.h - Class representing an explosion flash.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef EXPLODEFLASH_H_
#define EXPLODEFLASH_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

#include <string>

class Drawer;

///	Class representing an explosion flash.
class ExplodeFlash
{
  public:
	///	Constructor.
	ExplodeFlash(const TwoFloats& pos, const ThreeFloats& col);

	///	Updates the flash.
	void update();
	///	Draws the flash.
	void draw(shared_ptr<Drawer>& d);

	///	Returns true when the flash can be deleted.
	bool getDead() const {return bigTween.getFinished();};
  private:
	///	The position of the flash.
	TwoFloats position;
	///	The colour of the small flash.
	ThreeFloats colour;
	///	The size of the big flash.
	float bigSize;
	///	The size of the small flash.
	float smallSize;
	///	The tweener for the big flash.
	LogTweener<float> bigTween;
	///	The tweener for the small flash.
	LogTweener<float> smallTween;
};

#endif
