//	Explosion.cpp - Class representing an explosion.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "Explosion.h"
#include "Player.h"
#include "Random.h"

//------------------------------------------------------------------------------
Explosion::Explosion(const shared_ptr<Player>& player):
position(player->getPosition()),
radius(0.0f),
colour(player->getColour()),
alpha(1.0f),
radTween(radius, 128.0f, 0.05f),
alphaTween(alpha, alpha, 0.066f),
parent(player),
rayRotate(Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI),
raySize(0.0f),
rayTween(raySize, 1.0f, 0.066f)
{
	alphaTween.setIndex(1.0f);
}

//------------------------------------------------------------------------------
void Explosion::update()
{
	if(!radTween.getFinished())
	{
		radTween.tick();
		if(radTween.getFinished())
			alphaTween.reset(0.0f);
	}

	if(!alphaTween.getFinished())
		alphaTween.tick();

	if(!rayTween.getFinished())
		rayTween.tick();
	rayRotate += 0.01f;
}

//------------------------------------------------------------------------------
void Explosion::drawRays(shared_ptr<Drawer>& d)
{
	for(int i=0;i<10;++i)
	{
		float rotateVal(rayRotate + (((float)i/10.0f) * 2.0f * NIALL_PI));

		d->drawRotImage(L"ExplodeRay",
						TwoFloats(position.x - (32.0f * raySize),
								  position.y - (192.0f * raySize)),
						rotateVal,
						TwoFloats(32.0f, 192.0f),
						raySize,
						1.0f,
						alpha);
	}
}

//------------------------------------------------------------------------------
void Explosion::draw(shared_ptr<Drawer>& d)
{
	d->addImageToBatch(L"Shockwave",
					   position - (140.5f * (radius/128.0f)),
					   (radius/128.0f),
					   colour,
					   alpha * 0.66f);
}

//------------------------------------------------------------------------------
void Explosion::addPlayer(shared_ptr<Player>& player)
{
	hurtPlayers.push_back(player);
}
