//	Explosion.h - Class representing an explosion.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef EXPLOSION_H_
#define EXPLOSION_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

#include <vector>

class Drawer;
class Player;

///	Class representing an explosion.
class Explosion
{
  public:
	///	Constructor.
	Explosion(const shared_ptr<Player>& player);

	///	Updates the explosion.
	void update();
	///	Draws the explosion's rays.
	void drawRays(shared_ptr<Drawer>& d);
	///	Draws the explosion.
	void draw(shared_ptr<Drawer>& d);

	///	Adds a player to our list of player's we've already hurt.
	void addPlayer(shared_ptr<Player>& player);

	///	Returns the explosion's position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the explosion's radius.
	float getRadius() const {return radius;};
	///	Returns the explosion's parent.
	const shared_ptr<Player>& getParent() const {return parent;};
	///	Returns true if the explosion has worn itself out.
	bool getDead() const {return (alpha <= 0.0f);};
	///	Returns the list of players we've already hurt.
	const std::vector<shared_ptr<Player> >& getHurtPlayers() const {return hurtPlayers;};
  private:
	///	The explosion's position.
	TwoFloats position;
	///	The explosion's radius.
	float radius;
	///	The explosion's colour.
	ThreeFloats colour;
	///	The explosion's alpha value.
	float alpha;
	///	Used to tween the explosion's radius.
	LogTweener<float> radTween;
	///	Used to tween the explosion's alpha.
	LogTweener<float> alphaTween;
	///	The explosion's parent player.
	const shared_ptr<Player>& parent;

	///	Used to rotate the explosion's rays.
	float rayRotate;
	///	Used to expand the explosion's rays.
	float raySize;
	///	Used to expand the explosion's rays.
	LogTweener<float> rayTween;

	///	List of players we've already hurt.
	std::vector<shared_ptr<Player> > hurtPlayers;
};

#endif
