//	GamePlayState2.cpp - The timed gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "sound/GenericFileSounder.h"
#include "StarsGlobalData.h"
#include "drawing/Drawer.h"
#include "GamePlayState2.h"
#include "ExplodeFlash.h"
#include "MuzzleFlash.h"
#include "ScaleUpIcon.h"
#include "Application.h"
#include "SmokePuff.h"
#include "Explosion.h"
#include "Random.h"
#include "Player.h"
#include "Bullet.h"
#include "Wall2.h"
#include "Wall.h"

#include <iomanip>
#include <sstream>
#include "SDL.h"

//------------------------------------------------------------------------------
GameStateCreator<GamePlayState2> GamePlayState2::creator;

//------------------------------------------------------------------------------
GamePlayState2::GamePlayState2(Application& app, SounderPtr& sound):
GameState(app, sound),
mainTimer(0.0f),
mainTimerInc(1.0f),
numTriggered(0),
numAlive(0),
clockPos(TwoFloats(app.getDrawingAreaSize().x * 0.5f, 76.0f)),
clockPosTween(clockPos, clockPos, 0.05f),
clockScale(0.25f),
clockScaleTween(clockScale, clockScale, 0.05f),
clockAlpha(0.75f),
clockAlphaTween(clockAlpha, clockAlpha, 0.05f),
actionTick(1.0f/60.0f),
joinTimer(-1),
joinTimerFloat(0.0f),
joinMode(true),
explainFade(1.0f),
explainTween(explainFade, explainFade, 0.015f),
wallTimer(200),
//wallTimer(60),
waitingWallsTimer(1),
wallsWait(200),
gameOverCount(0),
numAI(0),
capsLock(false),
shakeAmp(0.0f),
shakeIndex(0.0f),
tutAction(3),
tutPlayer(new Player(0.0f, L"Player00", colourPalette)),
tutTimer(60),
tutCount(2),
sounder(dynamic_pointer_cast<GenericFileSounder>(sound))
{
	const TwoFloats& drawSize(app.getDrawingAreaSize());

	globalData = dynamic_pointer_cast<StarsGlobalData>(app.getGlobalData());
	holdMode = globalData->getHoldMode();

	clockPosTween.setIndex(1.0f);
	clockScaleTween.setIndex(1.0f);
	clockAlphaTween.setIndex(1.0f);

	backParticles.setSize(0.85f);
	backParticles.setLife(0.75f);

	frontParticles.setSize(0.33f);

	gridOffset.x = (drawSize.x - 1280.0f) * 0.5f;
	gridOffset.y = (drawSize.y - 720.0f) * 0.5f;

	tutPlayer->setPosition(gridOffset + TwoFloats(6.5f * 80.0f, (7.5f * 80.0f)));
	tutPlayer->start();
	tutPlayer->setAction(1);
	tutPlayer->pauseAction(true);

	explainTween.setIndex(1.0f);

	/*for(int i=0;i<5;++i)
	{
		walls.push_back(shared_ptr<Wall>(new Wall(TwoFloats((float)i * 80.0f,
															((drawSize.y * 0.5f) - 40.0f - (drawSize.x * 0.5f) + (float)i * 80.0f)),
										 colourPalette,
										 (float)i/12.0f)));
	}*/

	//Setup background.
	{
		int v = 0;
		
		for(int y=0;y<BackSizeY;++y)
		{
			for(int x=0;x<BackSizeX;++x)
			{
				float index(Random::getGlobal().getRandomFloat());

				//backGrid[y][x] = colourPalette.getBackgroundColour(index);

				if((x < 1) || (x > (BackSizeX-2)) || (y < 1) || (y > (BackSizeY-2)))
					backGrid[y][x] = colourPalette.getCloudColour(Random::getGlobal().getRandomFloat());
				else
					backGrid[y][x] = colourPalette.getBackgroundColour(Random::getGlobal().getRandomFloat());

				index = Random::getGlobal().getRandomFloat();
				if((x < 1) || (x > (BackSizeX-2)) || (y < 1) || (y > (BackSizeY-2)))
				{
					backTweeners.push_back(shared_ptr<Tweener<ThreeFloats> >(new Tweener<ThreeFloats>(backGrid[y][x],
																									  colourPalette.getCloudColour(index),
																									  0.005f + (Random::getGlobal().getRandomFloat() * 0.01f))));
				}
				else
				{
						backTweeners.push_back(shared_ptr<Tweener<ThreeFloats> >(new Tweener<ThreeFloats>(backGrid[y][x],
																									  colourPalette.getBackgroundColour(index),
																									  0.005f + (Random::getGlobal().getRandomFloat() * 0.01f))));
				}
			}
		}

		for(int y=0;y<(BackSizeY-1);++y)
		{
			for(int x=0;x<(BackSizeX-1);++x)
			{
				backgroundDrawer.setVertex(v,
										   TwoFloats((float)x*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
													 (float)y*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
										   backGrid[y][x],
										   1.0f);
				++v;
				backgroundDrawer.setVertex(v,
										   TwoFloats((float)(x+1)*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
													 (float)y*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
										   backGrid[y][x+1],
										   1.0f);
				++v;
				backgroundDrawer.setVertex(v,
										   TwoFloats((float)x*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
													 (float)(y+1)*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
										   backGrid[y+1][x],
										   1.0f);
				++v;
				backgroundDrawer.setVertex(v,
										   TwoFloats((float)(x+1)*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
													 (float)(y+1)*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
										   backGrid[y+1][x+1],
										   1.0f);
				++v;
			}
		}
	}

	//Setup start stuff.
	for(int y=0;y<GridSizeY;++y)
	{
		TwoFloats pos(gridOffset + 40.0f);

		pos.y += (y * 80.0f);
		for(int x=0;x<GridSizeX;++x)
		{
			startPositions.push_back(pos);

			pos.x += 80.0f;
		}
	}
	setupArms();

	for(int i=0;i<80;++i)
	{
		std::wstringstream tempstr;

		tempstr << L"Player";
		tempstr << std::setfill(L'0') << std::setw(2) << i;

		playerImages.push_back(tempstr.str());
	}

	app.setMousePos(drawSize * 0.5f);
	mousePos = drawSize * 0.5f;
}

//------------------------------------------------------------------------------
GamePlayState2::~GamePlayState2()
{
	
}

//------------------------------------------------------------------------------
void GamePlayState2::update()
{
	static int recursion = 0;
	const TwoFloats& drawSize(getApplication().getDrawingAreaSize());

	//Update background colours.
	for(unsigned int i=0;i<backTweeners.size();++i)
	{
		int x(i%BackSizeX);
		int y(i/BackSizeX);

		backTweeners[i]->tick();
		if(backTweeners[i]->getFinished())
		{
			if((x < 1) || (x > (BackSizeX-2)) || (y < 1) || (y > (BackSizeY-2)))
				backTweeners[i]->reset(colourPalette.getCloudColour(Random::getGlobal().getRandomFloat()));
			else
				backTweeners[i]->reset(colourPalette.getBackgroundColour(Random::getGlobal().getRandomFloat()));
		}
	}

	//Update screen shake.
	if(shakeAmp > 0.0f)
	{
		shakeAmp -= 0.05f;

		shakeIndex += 0.66f;
		if(shakeIndex > (2.0f * NIALL_PI))
			shakeIndex -= (2.0f * NIALL_PI);
	}

	if(mainTimer > 30.0f)
	{
		//Update walls.
		for(unsigned int i=0;i<walls.size();++i)
			walls[i]->update();
		for(unsigned int i=0;i<walls2.size();++i)
			walls2[i]->update();

		//Update muzzle flashes.
		for(int i=(muzzleFlashes.size()-1);i>=0;--i)
		{
			muzzleFlashes[i]->update();

			if(muzzleFlashes[i]->getDead())
				muzzleFlashes.erase(muzzleFlashes.begin() + i);
		}

		//Update explosion flashes.
		for(int i=(explodeFlashes.size()-1);i>=0;--i)
		{
			explodeFlashes[i]->update();

			if(explodeFlashes[i]->getDead())
				explodeFlashes.erase(explodeFlashes.begin() + i);
		}

		//Update smoke puffs.
		for(int i=(smokePuffs.size()-1);i>=0;--i)
		{
			smokePuffs[i]->update();

			if(smokePuffs[i]->getDead())
				smokePuffs.erase(smokePuffs.begin() + i);
		}

		//Update bullets.
		for(int i=(bullets.size()-1);i>=0;--i)
		{
			bullets[i]->update();

			TwoFloats pos(bullets[i]->getPosition());

			if((pos.x < -18.0f) || (pos.x > (drawSize.x + 18.0f)) ||
			   (pos.y < -18.0f) || (pos.y > (drawSize.y + 18.0f)))
			{
				bullets.erase(bullets.begin() + i);
			}
			else
			{
				TwoFloats dir(bullets[i]->getDirection().normal());

				frontParticles.trigger(bullets[i]->getPosition() - (dir * 8.0f),
									   dir,
									   bullets[i]->getColour(),
									   bullets[i]->getColour(),
									   0.5f,
									   4);
								   
			}
		}

		//Update explosions.
		for(int i=(explosions.size()-1);i>=0;--i)
		{
			explosions[i]->update();

			if(explosions[i]->getDead())
				explosions.erase(explosions.begin() + i);
		}

		//Update particles.
		backParticles.update();
		frontParticles.update();
	}

	//Update players.
	std::map<std::wstring, shared_ptr<Player> >::iterator it;
	for(it=players.begin();it!=players.end();++it)
	{
		it->second->setActionTick(actionTick);
		it->second->update(this);
	}

	//Update scale up icons.
	for(int i=(scaleUpIcons.size()-1);i>=0;--i)
	{
		scaleUpIcons[i]->update();
		if(scaleUpIcons[i]->getDead())
			scaleUpIcons.erase(scaleUpIcons.begin() + i);
	}

	//Update clock tweeners.
	if(!clockPosTween.getFinished())
		clockPosTween.tick();
	if(!clockScaleTween.getFinished())
		clockScaleTween.tick();
	if(!clockAlphaTween.getFinished())
		clockAlphaTween.tick();

	if(gameOverText.length() > 0)
	{
		if(gameOverCount > 0)
			--gameOverCount;

		recursion = 0;
		return;
	}

	//Update explanation text.
	if(!explainTween.getFinished())
		explainTween.tick();

	//Update join timer.
	if(joinMode)
	{
		if(joinTimer > 0)
		{
			uint32_t ticks(getApplication().getTimeSinceStart());

			joinTimerFloat += (float)(ticks - joinMillis)/1000.0f;
			if(joinTimerFloat > 1.0f)
			{
				joinTimerFloat -= 1.0f;
				--joinTimer;

				if(!joinTimer)
				{
					joinMode = false;

					lastTicks = getApplication().getTimeSinceStart();

					sounder->triggerSound(capsLock ? L"GongHi" : L"Gong", 1.66f);

					std::map<std::wstring, shared_ptr<Player> >::iterator it;
					for(it=players.begin();it!=players.end();++it)
						it->second->start();

					if((players.size() < 2) && (numAI < 1))
						numAI = 7;

					for(int i=0;((i<numAI) && (playerImages.size() > 0));++i)
					{
						int posIndex(Random::getGlobal().getRandomInt(startPositions.size()));
						int imageIndex(Random::getGlobal().getRandomInt(playerImages.size()));
						std::wstringstream tempstr;

						tempstr << L"AI" << i;
						players.insert(std::make_pair(tempstr.str(),
													  shared_ptr<Player>(new Player(startPositions[posIndex],
																					playerImages[imageIndex],
																					colourPalette))));
						players[tempstr.str()]->setAI();
						players[tempstr.str()]->setTimed();
						players[tempstr.str()]->start();
						++numAlive;

						startPositions.erase(startPositions.begin() + posIndex);
						playerImages.erase(playerImages.begin() + imageIndex);
					}

					sounder->triggerSound(capsLock ? L"NewPlayerHi" : L"NewPlayer", 2.0f);
				}
			}

			joinMillis = ticks;
		}
		else //Tutorial Stuff.
		{
			tutPlayer->update(this);

			--tutTimer;
			if(!tutTimer)
			{
				++tutCount;
				if(tutCount > 2)
				{
					++tutAction;
					tutAction %= 4;

					tutPlayer->setPosition(gridOffset + TwoFloats(6.5f * 80.0f, (7.5f * 80.0f)));

					if(tutAction < 3)
						tutPlayer->setAction((tutAction * 4) + 1);
					else
						tutPlayer->setAction(12);

					tutCount = 0;
				}

				tutPlayer->trigger(this);
				tutTimer = 60;
			}
		}
	}
	else if(joinTimer == 0)
	{
		uint32_t ticks(getApplication().getTimeSinceStart());

		joinTimerFloat += (float)(ticks - joinMillis)/1000.0f;
		if(joinTimerFloat > 1.0f)
		{
			--joinTimer;
			joinTimerFloat = 0.0f;
		}
		joinMillis = ticks;
	}

	if(!joinMode)
	{
		//Handle the clock.
		float inc(mainTimerInc);
		bool lastPhase(mainTimer > 30.0f);
		uint32_t ticks(getApplication().getTimeSinceStart());

		if(capsLock)
			inc *= 3.0f;

		if(mainTimer > 30.0f)
			inc = 1.0f;

		mainTimer += ((float)(ticks - lastTicks)/1000.0f) * inc;

		if((numTriggered >= numAlive) && (mainTimer < 30.0f))
			mainTimer += ((float)(ticks - lastTicks)/1000.0f) * 9.0f;

		lastTicks = ticks;

		if(mainTimer > 31.0f)
		{
			//Wait for explosions to dissipate before starting the next turn.
			if(explosions.size() < 1)
			{
				mainTimer = 0.0f;
				numTriggered = 0;

				std::map<std::wstring, shared_ptr<Player> >::iterator it;

				for(it=players.begin();it!=players.end();++it)
				{
					it->second->start();
					it->second->pauseAction(false);
				}

				clockPos = TwoFloats(drawSize.x * 0.5f, 76.0f);
				clockScale = 0.25f;
				clockAlpha = 0.75f;

				if(mainTimerInc < 15.0f)
					mainTimerInc += 0.1f;
			}
		}
		else if(mainTimer > 30.0f)
		{
			//Handle collision detection.
			collideExplosions();
			collideBullets();

			if(!lastPhase)
			{
				std::map<std::wstring, shared_ptr<Player> >::iterator it;

				for(it=players.begin();it!=players.end();++it)
				{
					if(it->second->getNumLives() > 0)
					{
						it->second->trigger(this);
						it->second->stop();
					}
				}

				clockAlphaTween.reset(0.0f);
			}
		}
	}

	//Determine if there's a winner.
	if(!joinMode)
	{
		int numDead(0);
		std::map<std::wstring, shared_ptr<Player> >::iterator it;

		for(it=players.begin();it!=players.end();++it)
		{
			if(it->second->getNumLives() < 1)
				++numDead;
		}
		if((players.size() - numDead) == 1)
		{
			gameOverText = L"WINNER!";
			gameOverCount = 60;

			for(it=players.begin();it!=players.end();++it)
			{
				if(it->second->getNumLives() > 0)
				{
					winner = it->second;
					winner->setPosition(drawSize * 0.5f);
					winner->stop();
					break;
				}
			}

			sounder->triggerSound(capsLock ? L"WinnerHi" : L"Winner", 1.66f);
		}
		else if((players.size() - numDead) < 1)
		{
			gameOverText = L"NOBODY WINS!";
			gameOverCount = 60;

			sounder->triggerSound(capsLock ? L"WinnerHi" : L"Winner", 1.66f);
		}
		else if(mainTimer > 30.0f)
		{
			//Update walls.
			if(waitingWalls.size() > 0)
			{
				if(!waitingWallsTimer)
				{
					int index(0);
					bool wallAdded(false);

					while(!wallAdded && (index < 1))//(index < (int)waitingWalls.size()))
					{
						bool canAddWall(true);
						const float radSq((37.0f + 36.0f) * (37.0f + 36.0f));
						std::map<std::wstring, shared_ptr<Player> >::iterator it;

						for(it=players.begin();it!=players.end();++it)
						{
							if((waitingWalls[index].getDistanceSquared(it->second->getPosition()) < radSq) ||
							   (waitingWalls[index].getDistanceSquared(it->second->getDestination()) < radSq))
							{
								if(it->second->getNumLives() > 0)
								{
									canAddWall = false;
									break;
								}
							}
						}

						if(canAddWall)
						{
							shared_ptr<Wall2> wall(new Wall2(waitingWalls[index], colourPalette));

							walls2.push_back(wall);
							waitingWalls.erase(waitingWalls.begin() + index);

							waitingWallsTimer = 3;
							if(waitingWalls.size() < 1)
							{
								wallTimer = wallsWait;

								wallsWait -= 15;
							}

							wallAdded = true;

							sounder->triggerSound(capsLock ? L"WallDropHi" : L"WallDrop");
						}
						else
							++index;
					}
				}
				else
					--waitingWallsTimer;
			}
			else
			{
				--wallTimer;
				if(!wallTimer)
				{
					int i((int)(wallPos.x/80.0f));

					if(i < 6)
					{
						//Fill out waitingWalls.
						//(top + bottom rows)
						for(int x=i;x<(GridSizeX-i);++x)
						{
							TwoFloats pos;

							pos.x = gridOffset.x + ((float)i * 80.0f) + ((float)(x-i) * 80.0f) + 40.0f;
							pos.y = (drawSize.y * 0.5f) - ((float)((GridSizeX/2)-i) * 80.0f);
							if(pos.isWithin(0.0f, drawSize))
								waitingWalls.push_back(pos);

							pos.x = gridOffset.x + ((float)i * 80.0f) + ((float)(x-i) * 80.0f) + 40.0f;
							pos.y = (drawSize.y * 0.5f) + ((float)((GridSizeX/2)-i) * 80.0f);
							if(pos.isWithin(0.0f, drawSize))
								waitingWalls.push_back(pos);
						}

						//(left + right columns)
						for(int y=i;y<(GridSizeX-i);++y)
						{
							TwoFloats pos;

							pos.x = gridOffset.x + ((float)i * 80.0f) + 40.0f;
							pos.y = (drawSize.y * 0.5f) - ((float)((GridSizeX/2)-i) * 80.0f) + ((float)(y-i) * 80.0f);
							if(pos.isWithin(0.0f, drawSize))
								waitingWalls.push_back(pos);

							pos.x = drawSize.x - gridOffset.x - ((float)i * 80.0f) - 80.0f + 40.0f;
							pos.y = (drawSize.y * 0.5f) - ((float)((GridSizeX/2)-i) * 80.0f) + ((float)(y-i) * 80.0f);
							if(pos.isWithin(0.0f, drawSize))
								waitingWalls.push_back(pos);
						}

						wallPos += 80.0f;
					}
				}
			}
		}
	}

	//Update actionTick.
	if(actionTick > (1.0f/60.0f))
	{
		actionTick -= 0.0033f;
		if(actionTick < (1.0f/60.0f))
			actionTick = (1.0f/60.0f);
	}

	if(!joinMode && (recursion < 2) && !globalData->getNoSpecials())
	{
		//Check if caps lock is on, triple-speed if so.
		SDL_Keymod modState = SDL_GetModState();

		if(modState & KMOD_CAPS)
		{
			if(!capsLock)
			{
				capsLock = true;
				//sounder->setPlaybackSpeed(2);
			}

			++recursion;
			update();
			--recursion;
		}
		else if(capsLock)
		{
			capsLock = false;
			//sounder->setPlaybackSpeed(1);
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::draw(DrawerPtr& d, float timestepInterp)
{
	const TwoFloats& drawSize(d->getDrawingAreaSize());

	if(!backgroundDrawer.getIsInitialised())
		backgroundDrawer.initialise(d);

	//Draw the background.
	{
		int v(0);

		for(int y=0;y<BackSizeY;++y)
		{
			for(int x=0;x<BackSizeX;++x)
			{
				backgroundDrawer.setVertex(v,
											TwoFloats((float)x*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
														(float)y*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
											backGrid[y][x],
											1.0f);
				++v;
				backgroundDrawer.setVertex(v,
											TwoFloats((float)(x+1)*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
														(float)y*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
											backGrid[y][x+1],
											1.0f);
				++v;
				backgroundDrawer.setVertex(v,
											TwoFloats((float)x*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
														(float)(y+1)*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
											backGrid[y+1][x],
											1.0f);
				++v;
				backgroundDrawer.setVertex(v,
											TwoFloats((float)(x+1)*(getDrawingAreaSize().x/(float)(BackSizeX-1)),
														(float)(y+1)*(getDrawingAreaSize().y/(float)(BackSizeY-1))),
											backGrid[y+1][x+1],
											1.0f);
				++v;
			}
		}
	}
	backgroundDrawer.draw(d);

	if(shakeAmp > 0.0f)
	{
		float shakeVal(sinf(shakeIndex));
		TwoFloats shakePos(shakeVal * fakeLog(shakeAmp) * 6.0f, 0.0f);

		d->translate(shakePos);
	}

	//Draw the grid.
	for(int y=0;y<GridSizeY;++y)
	{
		TwoFloats pos(gridOffset);

		if(y%2)
			pos.x += 80.0f;
		pos.y += 80.0f * (float)y;

		for(int x=0;x<GridSizeX;++x)
		{
			if(!(x%2))
			{
				/*d->fillRect(pos+TwoFloats(shadowX0 * 4.0f, shadowX1 * 4.0f),
							80.0f,
							1.0f,
							0.167f);*/
				d->fillRect(pos, 80.0f, 1.0f, 0.33f);
			}

			pos.x += 80.0f;
		}
	}

	//Draw explosion rays.
	for(unsigned int i=0;i<explosions.size();++i)
		explosions[i]->drawRays(d);

	//Draw players & bullets etc.
	d->clearBatch();

	backParticles.draw(d);

	for(unsigned int i=0;i<explosions.size();++i)
		explosions[i]->draw(d);

	for(unsigned int i=0;i<explodeFlashes.size();++i)
		explodeFlashes[i]->draw(d);

	for(unsigned int i=0;i<smokePuffs.size();++i)
		smokePuffs[i]->draw(d);

	for(unsigned int i=0;i<bullets.size();++i)
		bullets[i]->draw(d);

	for(unsigned int i=0;i<muzzleFlashes.size();++i)
		muzzleFlashes[i]->draw(d);

	std::map<std::wstring, shared_ptr<Player> >::iterator it;

	//Draw corpses first.
	for(it=players.begin();it!=players.end();++it)
	{
		if(it->second->getNumLives() < 1)
			it->second->draw(d);
	}
	for(it=players.begin();it!=players.end();++it)
	{
		if(it->second->getNumLives() > 0)
			it->second->draw(d);
	}

	if(joinMode && (joinTimer < 0))
	{
		std::wstring im[] = {L"TutMove",
							 L"TutShoot",
							 L"TutDoubleMove",
							 L"TutExplode"};

		d->addImageToBatch(L"StarBack",
						   gridOffset - 40.0f + 16.0f + TwoFloats(4.0f * 80.0f, 7.0f * 80.0f) + 2.0f,
						   (128.0f/56.0f) * 0.95f,
						   tutPlayer->getColour());
		d->addImageToBatch(im[tutAction],
						   gridOffset - 40.0f + 16.0f + TwoFloats(4.0f * 80.0f, 7.0f * 80.0f));

		tutPlayer->draw(d);
	}

	for(unsigned int i=0;i<walls.size();++i)
		walls[i]->draw(d);

	for(unsigned int i=0;i<walls2.size();++i)
		walls2[i]->draw(d);

	frontParticles.draw(d);

	for(unsigned int i=0;i<scaleUpIcons.size();++i)
		scaleUpIcons[i]->draw(d);

	d->drawBatch();

	//Draw walls.
	for(unsigned int i=0;i<walls.size();++i)
		walls[i]->draw(d);

	//Draw clock.
	if(!joinMode && (mainTimer < 30.5f))
	{
		float alpha(1.0f);
		float hand(((mainTimer)/30.0f) * NIALL_PI);

		if(mainTimer > 30.0f)
			hand = NIALL_PI;

		/*if((mainTimer > 30.0f) && (mainTimer < 30.5f))
			alpha = fakeLog((mainTimer - 30.0f) * 2.0f);*/

		if(clockScale <= 0.25f)
		{
			d->drawImage(L"ClockBack",
						 TwoFloats(drawSize.x * 0.5f, 80.0f) - 64.0f,
						 1.0f,
						 1.0f,
						 0.75f * alpha);
			d->drawRotImage(L"ClockHand",
							TwoFloats(drawSize.x * 0.5f, 80.0f) - 64.0f,
							hand,
							64.0f,
							1.0f,
							1.0f,
							0.75f * alpha);
		}
		else
		{
			d->drawImage(L"BigClockBack",
						 clockPos - (256.0f * clockScale),
						 clockScale,
						 1.0f,
						 clockAlpha);
			d->drawRotImage(L"BigClockHand",
							clockPos - (256.0f * clockScale),
							hand,
							256.0f,
							clockScale,
							1.0f,
							clockAlpha);
		}
	}

	if(joinMode)
	{
		//Draw player keys.
		std::map<std::wstring, shared_ptr<Player> >::iterator it;

		for(it=players.begin();it!=players.end();++it)
		{
			std::wstring upperString(it->first);

			for(std::wstring::iterator it2=upperString.begin();it2!=upperString.end();++it2)
				*it2 = ::towupper(*it2);

			d->drawText(upperString,
						L"SmallFont",
						it->second->getPosition() + TwoFloats(-40.0f, 17.0f),
						80.0f,
						TextLayout::Centre,
						1.0f,
						it->second->getColour());
		}

		//Draw explanation text.
		if(explainFade > 0.0f)
		{
			d->drawText(L"HOLD KEY UNTIL TIMER COUNTS DOWN",
						L"BigFont",
						explainLayout,
						TwoFloats((drawSize.x*0.5f) - 400.0f + 4.0f, (drawSize.y*0.5f) - 24.0f + 4.0f),
						800.0f,
						TextLayout::Centre,
						1.0f,
						0.0f,
						0.5f * explainFade);
			d->drawText(L"HOLD KEY UNTIL TIMER COUNTS DOWN",
						L"BigFont",
						explainLayout,
						TwoFloats((drawSize.x*0.5f) - 400.0f, (drawSize.y*0.5f) - 24.0f),
						800.0f,
						TextLayout::Centre,
						1.0f,
						1.0f,
						1.0f * explainFade);
		}

		//Draw countdown.
		if(joinTimer > 0)
		{
			std::wstringstream tempstr;

			tempstr << joinTimer;
			d->drawText(tempstr.str(),
						L"BiggerFont",
						TwoFloats(0.0f, 208.0f),
						drawSize.x,
						TextLayout::Centre,
						1.0f,
						0.0f,
						0.33f * fakeLog(1.0f - joinTimerFloat));
		}
	}
	else if(joinTimer == 0)
	{
		d->drawText(L"GO!",
					L"BiggerFont",
					TwoFloats(0.0f, 208.0f),
					drawSize.x,
					TextLayout::Centre,
					1.0f,
					0.0f,
					0.33f * fakeLog(1.0f - joinTimerFloat));
	}

	if(gameOverText.length() > 0)
	{
		d->drawText(gameOverText,
					L"BigFont",
					TwoFloats(4.0f, (drawSize.y * 0.5f) + 160.0f + 4.0f),
					drawSize.x,
					TextLayout::Centre,
					1.0f,
					0.0f,
					0.5f);
		d->drawText(gameOverText,
					L"BigFont",
					TwoFloats(0.0f, (drawSize.y * 0.5f) + 160.0f),
					drawSize.x,
					TextLayout::Centre,
					1.0f,
					1.0f,
					1.0f);
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::keyDown(const std::wstring& action,
							 const std::wstring& keyPressed,
							 int keyModifier)
{
	if(action == L"Escape")
		nextState(L"TitleScreen");
	else if(gameOverCount < 1)
	{
		if(gameOverText.length() > 0)
		{
			if(globalData->getExpoMode())
				nextState(L"GamePlayState2");
			else
				nextState(L"TitleScreen");
		}
		else if(joinMode)
		{
			if(keyPressed == L"f1")
				++numAI;
			/*else if(keyPressed == L"f2")
				holdMode = !holdMode;
			else if(keyPressed == L"f3")
			{
				if(sounder->getMusicIsPlaying())
					sounder->stopMusic();
				else
					sounder->playMusic(L"Music");
			}*/
			else if((playerImages.size() > 0) &&
					(startPositions.size() > 0) &&
					(keyPressed != L"capslock"))
			{
				int posIndex(Random::getGlobal().getRandomInt(startPositions.size()));
				int imageIndex(Random::getGlobal().getRandomInt(playerImages.size()));
				
				if((explainFade > 0.0f) && (explainTween.getFinished()))
				{
					explainTween.reset(0.0f);
					joinTimer = 10;
					joinMillis = getApplication().getTimeSinceStart();
				}

				players.insert(std::make_pair(keyPressed,
											  shared_ptr<Player>(new Player(startPositions[posIndex],
																			playerImages[imageIndex],
																			colourPalette))));
				players[keyPressed]->setTimed();
				joinKeys.push_back(keyPressed);
				++numAlive;

				startPositions.erase(startPositions.begin() + posIndex);
				playerImages.erase(playerImages.begin() + imageIndex);

				sounder->triggerSound(capsLock ? L"NewPlayerHi" : L"NewPlayer", 2.0f);
			}
		}
		else if(mainTimer < 30.0f)
		{
			std::map<std::wstring, shared_ptr<Player> >::iterator it;

			it = players.find(keyPressed);
			if(it != players.end())
			{
				if((it->second->getNumLives() > 0) && (!it->second->getPaused()))
				{
					std::wstring ims[] = {L"BigMove0",
										  L"TutMove",
										  L"BigMove2",
										  L"BigMove3",
										  L"BigShoot0",
										  L"TutShoot",
										  L"BigShoot2",
										  L"BigShoot3",
										  L"BigDoubleMove0",
										  L"TutDoubleMove",
										  L"BigDoubleMove2",
										  L"BigDoubleMove3",
										  L"TutExplode"};

					it->second->pauseAction(true);
					++numTriggered;

					if(numTriggered >= numAlive)
					{
						clockPosTween.reset(getApplication().getDrawingAreaSize() * 0.5f);
						clockScaleTween.reset(1.0f);
						clockAlphaTween.reset(1.0f);
					}

					if(capsLock)
						sounder->triggerSound(L"TimedPickHi", 1.33f);
					else
						sounder->triggerSound(L"TimedPick", 1.33f);

					addScaleUpIcon(ims[it->second->getAction()],
								   it->second->getPosition());
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::keyUp(const std::wstring& action,
						  const std::wstring& keyPressed,
						  int keyModifier)
{
	bool foundKey(false);

	for(unsigned int i=0;i<joinKeys.size();++i)
	{
		if(joinKeys[i] == keyPressed)
		{
			joinKeys.erase(joinKeys.begin() + i);
			foundKey = true;
			break;
		}
	}

	if(joinMode)
	{
		std::map<std::wstring, shared_ptr<Player> >::iterator it;

		it = players.find(keyPressed);
		if(it != players.end())
		{
			startPositions.push_back(it->second->getPosition());
			playerImages.push_back(it->second->getImage());

			players.erase(it);
			--numAlive;
		}
	}
	else if(holdMode && !foundKey)
	{
		std::map<std::wstring, shared_ptr<Player> >::iterator it;

		it = players.find(keyPressed);
		if(it != players.end())
		{
			if(it->second->getNumLives() > 0)
			{
				it->second->trigger(this);
				it->second->pauseAction(false);
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::mouseMove(const TwoFloats& pos)
{
	if(!joinMode && !globalData->getNoSpecials() && (gameOverText.length() < 1))
	{
		float dist(mousePos.getDistance(pos));

		if(dist > 16.0f)
			dist = 16.0f;
		dist /= 16.0f;
		dist = fakeExp(dist) * 0.25f;

		if(dist > actionTick)
		{
			actionTick = dist;

			std::map<std::wstring, shared_ptr<Player> >::iterator it;
			for(it=players.begin();it!=players.end();++it)
				it->second->setActionTick(actionTick);

			sounder->triggerSound(capsLock ? L"TrackballHi" : L"Trackball", 1.66f);
		}

		getApplication().setMousePos(getApplication().getDrawingAreaSize() * 0.5f);
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::joyMove(const std::wstring& action,
							float value,
							int joystick,
							int axis)
{
	
}

//------------------------------------------------------------------------------
bool GamePlayState2::getSpaceIsFree(const TwoFloats& pos,
								   const TwoFloats& dir,
								   int& recursion)
{
	bool retval(true);
	std::map<std::wstring, shared_ptr<Player> >::iterator it;
	const TwoFloats& drawSize(getApplication().getDrawingAreaSize());

	if((pos.x < 0.0f) || (pos.x >= drawSize.x) ||
	   (pos.y < 0.0f) || (pos.y >= drawSize.y))
	{
		return false;
	}

	//Test against walls.
	for(unsigned int i=0;i<walls2.size();++i)
	{
		if((walls2[i]->getPosition().getDistanceSquared(pos) < (16.0f * 16.0f)) ||
		   (walls2[i]->getDestination().getDistanceSquared(pos) < (16.0f * 16.0f)))
		{
			retval = false;
			break;
		}
	}
	
	if(retval)
	{
		//Test against players.
		for(it=players.begin();it!=players.end();++it)
		{
			TwoFloats tempPos;

			if(it->second->getNumLives() < 1)
				continue;

			if(it->second->getMoving())
				tempPos = it->second->getDestination();
			else
				tempPos = it->second->getPosition();

			if(tempPos.getDistanceSquared(pos) < (32.0f * 32.0f))
			{
				if(it->second->getMoving())
					retval = false;
				else
				{
					++recursion;
					if(recursion < 2)
					{
						if(getSpaceIsFree(pos+dir, dir, recursion))
							it->second->push(pos+dir);
						else
							retval = false;
					}
					else
						retval = false;
				}
				break;
			}
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
bool GamePlayState2::getWall(const TwoFloats& pos, int dir) const
{
	bool retval(false);
	TwoFloats tempPos(pos);
	const TwoFloats& drawSize(getApplication().getDrawingAreaSize());
	const TwoFloats dirs[] = {TwoFloats(0.0f, -80.0f),
							  TwoFloats(80.0f, 0.0f),
							  TwoFloats(00.0f, 80.0f),
							  TwoFloats(-80.0f, 0.0f)};

	tempPos += dirs[dir];

	if((tempPos.x < 0.0f) || (tempPos.x >= drawSize.x) ||
	   (tempPos.y < 0.0f) || (tempPos.y >= drawSize.y))
	{
		retval = true;
	}
	else
	{
		for(unsigned int i=0;i<walls2.size();++i)
		{
			if((walls2[i]->getPosition().getDistanceSquared(tempPos) < (16.0f * 16.0f)) ||
			   (walls2[i]->getDestination().getDistanceSquared(tempPos) < (16.0f * 16.0f)))
			{
				retval = true;
				break;
			}
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
void GamePlayState2::addBullet(int dir, Player *player)
{
	//First find the player.
	if(player == tutPlayer.get())
	{
		TwoFloats dirs[] = {TwoFloats(0.0f, -4.0f),
							TwoFloats(4.0f, 0.0f),
							TwoFloats(0.0f, 4.0f),
							TwoFloats(-4.0f, 0.0f)};
		shared_ptr<Bullet> bullet(new Bullet(dir, tutPlayer));

		bullets.push_back(bullet);

		addMuzzleFlash(player->getPosition() - dirs[dir],
						player->getColour(),
						dir);

		backParticles.trigger(player->getPosition() - dirs[dir] * 6.0f,
								dirs[dir] * -0.0625f,
								1.0f,
								player->getColour() + 0.15f);
		backParticles.trigger(player->getPosition() - dirs[dir] * 6.0f,
								dirs[dir] * -0.0625f,
								1.0f,
								1.0f);

		frontParticles.trigger(player->getPosition() + dirs[dir] * 5.0f,
								0.0f,
								1.0f,
								1.0f);
		frontParticles.trigger(player->getPosition() + dirs[dir] * 5.0f,
								0.0f,
								player->getColour() + 1.2f,
								1.0f);

		sounder->triggerSound(capsLock ? L"ShootHi" : L"Shoot", 1.66f);
	}
	else
	{
		std::map<std::wstring, shared_ptr<Player> >::iterator it;

		for(it=players.begin();it!=players.end();++it)
		{
			if(it->second.get() == player)
			{
				TwoFloats dirs[] = {TwoFloats(0.0f, -4.0f),
									TwoFloats(4.0f, 0.0f),
									TwoFloats(0.0f, 4.0f),
									TwoFloats(-4.0f, 0.0f)};
				shared_ptr<Bullet> bullet(new Bullet(dir, it->second));

				bullets.push_back(bullet);

				addMuzzleFlash(player->getPosition() - dirs[dir],
							   player->getColour(),
							   dir);

				backParticles.trigger(player->getPosition() - dirs[dir] * 6.0f,
									  dirs[dir] * -0.0625f,
									  1.0f,
									  player->getColour() + 0.15f);
				backParticles.trigger(player->getPosition() - dirs[dir] * 6.0f,
									  dirs[dir] * -0.0625f,
									  1.0f,
									  1.0f);

				frontParticles.trigger(player->getPosition() + dirs[dir] * 5.0f,
									   0.0f,
									   1.0f,
									   1.0f);
				frontParticles.trigger(player->getPosition() + dirs[dir] * 5.0f,
									   0.0f,
									   player->getColour() + 1.2f,
									   1.0f);

				sounder->triggerSound(capsLock ? L"ShootHi" : L"Shoot", 1.66f);

				break;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::addExplosion(Player *player)
{
	if(player == tutPlayer.get())
	{
		shared_ptr<Explosion> explosion(new Explosion(tutPlayer));

		explosions.push_back(explosion);

		addExplodeFlash(player->getPosition(), player->getColour());
		addSmokePuffs(player->getPosition());

		frontParticles.trigger(player->getPosition(),
								0.0f,
								player->getColour(),
								1.0f);
		frontParticles.trigger(player->getPosition(),
								0.0f,
								1.0f,
								1.0f);
		frontParticles.trigger(player->getPosition(),
								0.0f,
								player->getColour() + 1.1f,
								1.0f);

		shakeAmp = 1.0f;

		sounder->triggerSound(capsLock ? L"ExplodeHi" : L"Explode", 2.0f);
	}
	else
	{
		//First find the player.
		std::map<std::wstring, shared_ptr<Player> >::iterator it;

		for(it=players.begin();it!=players.end();++it)
		{
			if(it->second.get() == player)
			{
				shared_ptr<Explosion> explosion(new Explosion(it->second));

				explosions.push_back(explosion);

				addExplodeFlash(player->getPosition(), player->getColour());
				addSmokePuffs(player->getPosition());

				frontParticles.trigger(player->getPosition(),
									   0.0f,
									   player->getColour(),
									   1.0f);
				frontParticles.trigger(player->getPosition(),
									   0.0f,
									   1.0f,
									   1.0f);
				frontParticles.trigger(player->getPosition(),
									   0.0f,
									   player->getColour() + 1.1f,
									   1.0f);

				shakeAmp = 1.0f;

				sounder->triggerSound(capsLock ? L"ExplodeHi" : L"Explode", 2.0f);
				break;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::addMuzzleFlash(const TwoFloats& pos,
								   const ThreeFloats& col,
								   int dir)
{
	shared_ptr<MuzzleFlash> flash(new MuzzleFlash(pos, col, dir));

	muzzleFlashes.push_back(flash);
}

//------------------------------------------------------------------------------
void GamePlayState2::addExplodeFlash(const TwoFloats& pos,
								    const ThreeFloats& col)
{
	shared_ptr<ExplodeFlash> flash(new ExplodeFlash(pos, col));

	explodeFlashes.push_back(flash);
}

//------------------------------------------------------------------------------
void GamePlayState2::addSmokePuffs(const TwoFloats& pos)
{
	const int numPuffs(8 + Random::getGlobal().getRandomInt(8));
	float angle(Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI);
	float angleInc((2.0f * NIALL_PI)/(float)numPuffs);

	for(int i=0;i<numPuffs;++i)
	{
		shared_ptr<SmokePuff> puff(new SmokePuff(pos, TwoFloats(sinf(angle),
																cosf(angle))));

		smokePuffs.push_back(puff);

		angle += angleInc + ((Random::getGlobal().getRandomFloat() * angleInc) - (angleInc * 0.5f));
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::addScaleUpIcon(const std::wstring& im,
									const TwoFloats& pos)
{
	scaleUpIcons.push_back(shared_ptr<ScaleUpIcon>(new ScaleUpIcon(im, pos)));
}

//------------------------------------------------------------------------------
void GamePlayState2::aiChoiceMade(Player *player)
{
	if((player->getNumLives() > 0) && (!player->getPaused()))
	{
		std::wstring ims[] = {L"BigMove0",
							  L"TutMove",
							  L"BigMove2",
							  L"BigMove3",
							  L"BigShoot0",
							  L"TutShoot",
							  L"BigShoot2",
							  L"BigShoot3",
							  L"BigDoubleMove0",
							  L"TutDoubleMove",
							  L"BigDoubleMove2",
							  L"BigDoubleMove3",
							  L"TutExplode"};

		player->pauseAction(true);
		++numTriggered;

		if(numTriggered >= numAlive)
		{
			clockPosTween.reset(getApplication().getDrawingAreaSize() * 0.5f);
			clockScaleTween.reset(1.0f);
			clockAlphaTween.reset(1.0f);
		}

		if(capsLock)
			sounder->triggerSound(L"TimedPickHi", 1.33f);
		else
			sounder->triggerSound(L"TimedPick", 1.33f);

		addScaleUpIcon(ims[player->getAction()],
					   player->getPosition());
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::setupArms()
{
	const int NumTypes = 5;
	std::wstring arms[NumTypes][4] = {{L"ArmCircle0", L"ArmCircle1", L"ArmCircle2", L"ArmCircle3"},
									  {L"ArmCog0", L"ArmCog1", L"ArmCog2", L"ArmCog3"},
									  {L"ArmSpine0", L"ArmSpine1", L"ArmSpine2", L"ArmSpine3"},
									  {L"ArmSpiral0", L"ArmSpiral1", L"ArmSpiral2", L"ArmSpiral3"},
									  {L"ArmTri0", L"ArmTri1", L"ArmTri2", L"ArmTri3"}};
	std::vector<int> layouts[15];

	//Fill out layouts.
	int index(0);
	//Single arm.
	for(int i=0;i<4;++i)
	{
		layouts[index].push_back(i);
		++index;
	}
	//Double arm.
	for(int i=0;i<4;++i)
	{
		for(int j=(i+1);j<4;++j)
		{
			layouts[index].push_back(i);
			layouts[index].push_back(j);
			++index;
		}
	}
	//Triple arm.
	layouts[index].push_back(0); layouts[index].push_back(1); layouts[index].push_back(2);
	++index;
	layouts[index].push_back(0); layouts[index].push_back(2); layouts[index].push_back(3);
	++index;
	layouts[index].push_back(1); layouts[index].push_back(2); layouts[index].push_back(3);
	++index;
	layouts[index].push_back(0); layouts[index].push_back(1); layouts[index].push_back(3);
	++index;
	//All arms.
	layouts[index].push_back(0); layouts[index].push_back(1); layouts[index].push_back(2); layouts[index].push_back(3);

	/*for(int i=0;i<NumTypes;++i)
	{
		for(int j=0;j<15;++j)
		{
			armLayouts.push_back(std::vector<std::wstring>());

			for(unsigned int k=0;k<layouts[j].size();++k)
				armLayouts.back().push_back(arms[i][layouts[j][k]]);
		}
	}*/
}

//------------------------------------------------------------------------------
void GamePlayState2::collideExplosions()
{
	for(unsigned int i=0;i<explosions.size();++i)
	{
		const TwoFloats& expPos(explosions[i]->getPosition());
		const float expRad(explosions[i]->getRadius());

		//First collide with bullets.
		for(int j=(bullets.size()-1);j>=0;--j)
		{
			float dist(expPos.getDistanceSquared(bullets[j]->getPosition()));

			if(dist < ((expRad + 12.0f) * (expRad + 12.0f)))
			{
				addExplodeFlash(bullets[j]->getPosition(), bullets[j]->getColour());

				frontParticles.trigger(bullets[j]->getPosition(),
									   0.0f,
									   bullets[j]->getColour(),
									   1.0f);
				frontParticles.trigger(bullets[j]->getPosition(),
									   0.0f,
									   1.0f,
									   1.0f);
				frontParticles.trigger(bullets[j]->getPosition(),
									   0.0f,
									   bullets[j]->getColour() + 1.1f,
									   1.0f);
				shakeAmp = 1.0f;

				bullets.erase(bullets.begin() + j);

				sounder->triggerSound(capsLock ? L"BulletHit2Hi" : L"Bullet2Hit", 1.66f);
				sounder->triggerSound(capsLock ? L"BulletHitHi" : L"BulletHit", 1.66f);
				sounder->triggerSound(capsLock ? L"ShootHi" : L"Shoot", 1.66f);

				break;
			}
		}

		//Now collide with players.
		std::map<std::wstring, shared_ptr<Player> >::iterator it;

		for(it=players.begin();it!=players.end();++it)
		{
			if((it->second->getNumLives() > 0) && (it->second != explosions[i]->getParent()))
			{
				float dist(expPos.getDistanceSquared(it->second->getPosition()));

				if(dist < ((expRad + 26.0f) * (expRad + 26.0f)))
				{
					//Check we've not already hurt this player.
					const std::vector<shared_ptr<Player> >& hurtPlayers(explosions[i]->getHurtPlayers());
					bool alreadyHurt(false);

					for(unsigned int j=0;j<hurtPlayers.size();++j)
					{
						if(it->second == hurtPlayers[j])
						{
							alreadyHurt = true;
							break;
						}
					}

					if(!alreadyHurt)
					{
						it->second->hurt();
						explosions[i]->addPlayer(it->second);
						shakeAmp = 1.0f;

						addExplodeFlash(it->second->getPosition(),
										it->second->getColour());

						frontParticles.trigger(it->second->getPosition(),
											   0.0f,
											   it->second->getColour(),
											   1.0f);
						frontParticles.trigger(it->second->getPosition(),
											   0.0f,
											   1.0f,
											   1.0f);
						frontParticles.trigger(it->second->getPosition(),
											   0.0f,
											   it->second->getColour() + 1.1f,
											   1.0f);

						if(it->second->getNumLives() > 0)
							sounder->triggerSound(capsLock ? L"HitHi" : L"Hit", 1.66f);
						else
						{
							sounder->triggerSound(capsLock ? L"DieHi" : L"Die", 1.66f);
							--numAlive;
						}
					}
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState2::collideBullets()
{
	//First collide with other bullets.
	for(int i=(bullets.size()-1);i>=0;--i)
	{
		const TwoFloats& bulletPos(bullets[i]->getPosition());

		for(int j=(bullets.size()-1);j>i;--j)
		{
			float dist(bulletPos.getDistanceSquared(bullets[j]->getPosition()));

			if(dist < ((12.0f + 12.0f) * (12.0f + 12.0f)))
			{
				TwoFloats effectPos(bulletPos + ((bullets[j]->getPosition() - bulletPos) * 0.5f));
				addExplodeFlash(effectPos, bullets[i]->getColour());

				frontParticles.trigger(effectPos,
									   0.0f,
									   bullets[i]->getColour(),
									   1.0f);
				frontParticles.trigger(effectPos,
									   0.0f,
									   1.0f,
									   1.0f);
				frontParticles.trigger(effectPos,
									   0.0f,
									   bullets[j]->getColour() + 1.1f,
									   1.0f);
				shakeAmp = 1.0f;

				bullets.erase(bullets.begin() + j);
				bullets.erase(bullets.begin() + i);

				sounder->triggerSound(capsLock ? L"BulletHit2Hi" : L"BulletHit2", 1.66f);
				sounder->triggerSound(capsLock ? L"BulletHitHi" : L"BulletHit", 1.66f);
				sounder->triggerSound(capsLock ? L"ShootHi" : L"Shoot", 1.66f);

				break;
			}
		}
	}

	//Then collide with walls.
	for(int i=(bullets.size()-1);i>=0;--i)
	{
		const TwoFloats& bulletPos(bullets[i]->getPosition());

		for(unsigned int j=0;j<walls2.size();++j)
		{
			float dist(bulletPos.getDistanceSquared(walls2[j]->getPosition()));

			if(dist < ((12.0f + 37.0f) * (12.0f + 37.0f)))
			{
				addExplodeFlash(bullets[i]->getPosition(),
								bullets[i]->getColour());

				frontParticles.trigger(bullets[i]->getPosition(),
									   0.0f,
									   bullets[i]->getColour(),
									   1.0f);
				frontParticles.trigger(bullets[i]->getPosition(),
									   0.0f,
									   1.0f,
									   1.0f);
				frontParticles.trigger(bullets[i]->getPosition(),
									   0.0f,
									   bullets[i]->getColour() + 1.1f,
									   1.0f);

				bullets.erase(bullets.begin() + i);

				sounder->triggerSound(capsLock ? L"BulletHitHi" : L"BulletHit", 1.66f);

				break;
			}
		}
	}

	//Now collide with players.
	for(int i=(bullets.size()-1);i>=0;--i)
	{
		std::map<std::wstring, shared_ptr<Player> >::iterator it;
		const TwoFloats& bulletPos(bullets[i]->getPosition());

		for(it=players.begin();it!=players.end();++it)
		{
			if((it->second->getNumLives() > 0) && (it->second != bullets[i]->getParent()))
			{
				float dist(bulletPos.getDistanceSquared(it->second->getPosition()));

				if(dist < ((12.0f + 26.0f) * (12.0f + 26.0f)))
				{
					it->second->hurt();
					shakeAmp = 1.0f;

					addExplodeFlash(it->second->getPosition(),
									it->second->getColour());

					frontParticles.trigger(it->second->getPosition(),
										   0.0f,
										   it->second->getColour(),
										   1.0f);
					frontParticles.trigger(it->second->getPosition(),
										   0.0f,
										   1.0f,
										   1.0f);
					frontParticles.trigger(it->second->getPosition(),
										   0.0f,
										   it->second->getColour() + 1.1f,
										   1.0f);

					bullets.erase(bullets.begin() + i);

					if(it->second->getNumLives() > 0)
						sounder->triggerSound(capsLock ? L"HitHi" : L"Hit", 1.66f);
					else
					{
						sounder->triggerSound(capsLock ? L"DieHi" : L"Die", 1.66f);
						--numAlive;
					}
					break;
				}
			}
		}
	}
}
