//	MuzzleFlash.cpp - Class representing a muzzle flash.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "MuzzleFlash.h"

//------------------------------------------------------------------------------
MuzzleFlash::MuzzleFlash(const TwoFloats& pos,
						 const ThreeFloats& col,
						 int dir):
position(pos),
colour(col),
bigSize(0.33f),
smallSize(0.25f),
bigTween(bigSize, 1.0f, 0.066f),
smallTween(smallSize, 1.0f, 0.09f)
{
	std::wstring bigIm[] = {L"MuzzleFlashBig0",
							L"MuzzleFlashBig1",
							L"MuzzleFlashBig2",
							L"MuzzleFlashBig3"};
	std::wstring smallIm[] = {L"MuzzleFlashSmall0",
							  L"MuzzleFlashSmall1",
							  L"MuzzleFlashSmall2",
							  L"MuzzleFlashSmall3"};

	bigImage = bigIm[dir];
	smallImage = smallIm[dir];
}

//------------------------------------------------------------------------------
void MuzzleFlash::update()
{
	if(!bigTween.getFinished())
		bigTween.tick();
	if(!smallTween.getFinished())
		smallTween.tick();
}

//------------------------------------------------------------------------------
void MuzzleFlash::draw(shared_ptr<Drawer>& d)
{
	d->addImageToBatch(smallImage,
					   position - (smallSize * 56.0f),
					   smallSize,
					   colour);
	d->addImageToBatch(bigImage,
					   position - (bigSize * 56.0f),
					   bigSize,
					   1.0f);
}
