//	ParticleGenerator.cpp - Simple particle generator.
//  ----------------------------------------------------------------------------
//	This file is part of 'RUMBLECURSE', a rumbling local multiplayer game.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ParticleGenerator.h"
#include "drawing/Drawer.h"
#include "MiscHelper.h"
#include "Random.h"

#include <cmath>

//------------------------------------------------------------------------------
ParticleGenerator::ParticleGenerator():
minSpeed(0.5f),
maxSpeed(6.0f),
colour(1.0f),
fadeColour(1.0f),
size(1.0f),
life(1.0f)
{

}

//------------------------------------------------------------------------------
ParticleGenerator::~ParticleGenerator()
{

}

//------------------------------------------------------------------------------
void ParticleGenerator::update()
{
	int i;

	for(i=(particles.size()-1);i>=0;--i)
	{
		particles[i].position += particles[i].direction * particles[i].speed * particles[i].speedMod;
		particles[i].direction += particles[i].gravity;
		particles[i].direction += gravity;
		particles[i].direction = particles[i].direction.normal();
		particles[i].life -= (particles[i].speed/(maxSpeed*maxSpeed)) * 0.75f;
		if(particles[i].life <= 0.0f)
			particles.erase(particles.begin()+i);
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::draw(shared_ptr<Drawer>& d, float timestepInterp)
{
	unsigned int i;
	//const TwoFloats size(0.5f);
	ThreeFloats col;

	for(i=0;i<particles.size();++i)
	{
		col = particles[i].fadeColour + (fakeLog(particles[i].life) * (particles[i].colour-particles[i].fadeColour));

		d->addImageToBatch(L"Blob",
						   particles[i].position-(16.0f*size*fakeLog(particles[i].life)),
						   size * fakeLog(particles[i].life),
						   col,
						   fakeLog(particles[i].life));
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::trigger(const TwoFloats& pos,
								const TwoFloats& grav,
								const ThreeFloats& col,
								const ThreeFloats& fadeCol,
								float speed,
								int numParticles)
{
	int i;
	float dir;

	for(i=0;i<numParticles;++i)
	{
		particles.push_back(Particle());
		particles.back().position = pos;
		dir = Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI;
		particles.back().direction.x = sinf(dir);
		particles.back().direction.y = cosf(dir);
		particles.back().gravity = grav;
		particles.back().speed = minSpeed + (Random::getGlobal().getRandomFloat() * (maxSpeed-minSpeed));
		particles.back().speedMod = speed;
		particles.back().life = life;
		particles.back().colour = col + ((Random::getGlobal().getRandomFloat() * 0.2f) - 0.1f);
		particles.back().fadeColour = fadeCol + ((Random::getGlobal().getRandomFloat() * 0.2f) - 0.1f);
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::setGravity(const TwoFloats& val)
{
	gravity = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setColour(const ThreeFloats& val)
{
	colour = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setFadeColour(const ThreeFloats& val)
{
	fadeColour = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setSize(float val)
{
	size = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setLife(float val)
{
	life = val;
}
