//	RainbowPalette.cpp - The original rainbow palette generator.
//  ----------------------------------------------------------------------------
//	This file is part of 'Hummingbird', an audiovisual ecosystem thing.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "RainbowPalette.h"
#include "ColourHelpers.h"

//------------------------------------------------------------------------------
ThreeFloats RainbowPalette::getColour(float val) const
{
	const float yellowPos((1.0f/6.0f) - (1.0f/24.0f));
	const float cyanPos(0.5f + (1.0f/24.0f));

	if((val > yellowPos) && (val < cyanPos))
	{
		val += (cyanPos-yellowPos);
		if(val > 1.0f)
			val -= 1.0f;
	}

	return ColourHelpers::hslToRgb(ThreeFloats(val, 0.8f, 0.66f));
}

//------------------------------------------------------------------------------
ThreeFloats RainbowPalette::getOutlineColour(float val) const
{
	ThreeFloats retval(val+0.8f, 0.9f, 0.66f);

	if(retval.x > 1.0f)
		retval.x -= 1.0f;

	return ColourHelpers::hslToRgb(retval);
}

//------------------------------------------------------------------------------
ThreeFloats RainbowPalette::getContrastingColour(float val) const
{
	ThreeFloats retval(val+0.33f, 0.9f, 0.66f);

	if(retval.x > 1.0f)
		retval.x -= 1.0f;

	return ColourHelpers::hslToRgb(retval);
}

//------------------------------------------------------------------------------
ThreeFloats RainbowPalette::getBackgroundColour(float val) const
{
	return ColourHelpers::hslToRgb(ThreeFloats(val, 0.8f, 0.66f));
}

//------------------------------------------------------------------------------
ThreeFloats RainbowPalette::getCloudColour(float val) const
{
	return ColourHelpers::hslToRgb(ThreeFloats(val, 1.0f, 0.95f));
}
