//	RainbowPalette.h - The original rainbow palette generator.
//  ----------------------------------------------------------------------------
//	This file is part of 'Hummingbird', an audiovisual ecosystem thing.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef RAINBOWPALETTE_H_
#define RAINBOWPALETTE_H_

#include "ColourPalette.h"

///	The original rainbow palette generator.
class RainbowPalette : public ColourPalette
{
  public:
	///	Returns a copy of this palette.
	ColourPalette *clone() const {return new RainbowPalette();};

	///	Returns the colour at point *val* in the palette.
	ThreeFloats getColour(float val) const;
	///	Returns a colour that can be used as an outline for the colour at point *val* in the palette.
	ThreeFloats getOutlineColour(float val) const;
	///	Returns a colour that will contrast with the colour at point *val* in the palette.
	ThreeFloats getContrastingColour(float val) const;
	///	Returns a background colour at point *val* in the palette.
	ThreeFloats getBackgroundColour(float val) const;
	///	Returns a cloud colour at point *val* in the palette.
	ThreeFloats getCloudColour(float val) const;

	///	Returns how rare this palette should be.
	float getRarity() const {return 0.1f;};
};

#endif
