//	ScaleUpIcon.cpp - Class which scales up an image then dies.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "ScaleUpIcon.h"

//------------------------------------------------------------------------------
ScaleUpIcon::ScaleUpIcon(const std::wstring& im, const TwoFloats pos):
image(im),
position(pos),
scale(0.0f),
alpha(1.0f),
scaleTween(scale, 1.0f, 0.02f),
alphaTween(alpha, 0.0f, 0.02f)
{

}

//------------------------------------------------------------------------------
void ScaleUpIcon::update()
{
	if(!scaleTween.getFinished())
		scaleTween.tick();
	if(!alphaTween.getFinished())
		alphaTween.tick();
}

//------------------------------------------------------------------------------
void ScaleUpIcon::draw(shared_ptr<Drawer>& d)
{
	d->addImageToBatch(image,
					   position - (64.0f * scale),
					   scale,
					   1.0f,
					   alpha);
}
