//	ScaleUpIcon.h - Class which scales up an image then dies.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SCALEUPICON_H_
#define SCALEUPICON_H_

#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

#include <string>

class Drawer;

///	Class which scales up an image then dies.
class ScaleUpIcon
{
  public:
	///	Constructor.
	ScaleUpIcon(const std::wstring& im, const TwoFloats pos);

	///	Scales up the image.
	void update();
	///	Draws the image.
	void draw(shared_ptr<Drawer>& d);

	///	Returns true if we can be deleted.
	bool getDead() const {return alphaTween.getFinished();};
  private:
	///	The image.
	std::wstring image;
	///	The image's position.
	TwoFloats position;
	///	The image's scale.
	float scale;
	///	The image's alpha.
	float alpha;
	///	Tweens scale.
	Tweener<float> scaleTween;
	///	Tweens the image's alpha.
	ExpTweener<float> alphaTween;
};

#endif
