//	SmokePuff.cpp - Class representing a single puff of smoke.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "SmokePuff.h"
#include "Random.h"

//------------------------------------------------------------------------------
SmokePuff::SmokePuff(const TwoFloats& pos, const TwoFloats& dir):
position(pos),
direction(dir),
rotDir((Random::getGlobal().getRandomFloat() < 0.5f) ? -1.0f : 1.0f),
speed(0.8f + (Random::getGlobal().getRandomFloat() * 0.2f)),
alpha(1.0f),
alphaTween(alpha, 0.0f, 0.01f),
x0(0.0f),
x1(1.0f),
coeff(2.0f*sinf(1.0f/60.0f) * rotDir)
{
	alphaTween.setDelay(Random::getGlobal().getRandomFloat() * 0.5f);
	alphaTween.reset(0.0f);

	vertices[0] = -25.5f;
	vertices[1].x = 25.5f;
	vertices[1].y = -25.5f;
	vertices[2].x = -25.5f;
	vertices[2].y = 25.5f;
	vertices[3] = 25.5f;
}

//------------------------------------------------------------------------------
void SmokePuff::update()
{
	x0 -= (coeff * x1);
	x1 += (coeff * x0);

	position += direction * speed * alpha;

	if(!alphaTween.getFinished())
		alphaTween.tick();
}

//------------------------------------------------------------------------------
void SmokePuff::draw(shared_ptr<Drawer>& d)
{
	array<TwoFloats, 4> tempArr;

	for(int i=0;i<4;++i)
	{
		tempArr[i].x = vertices[i].x * x0 - vertices[i].y * x1;
		tempArr[i].y = vertices[i].x * x1 + vertices[i].y * x0;

		tempArr[i] += position;
	}
	d->addImageVerticesToBatch(L"Smoke",
							   tempArr,
							   1.0f,
							   alpha * 0.5f);
}
