//	SmokePuff.h - Class representing a single puff of smoke.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SMOKEPUFF_H_
#define SMOKEPUFF_H_

#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

class Drawer;

///	Class representing a single puff of smoke.
class SmokePuff
{
  public:
	///	Constructor.
	SmokePuff(const TwoFloats& pos, const TwoFloats& dir);

	///	Updates the puff.
	void update();
	///	Draws the puff.
	void draw(shared_ptr<Drawer>& d);

	///	Returns true if the puff can be deleted.
	bool getDead() const {return alphaTween.getFinished();};
  private:
	///	The puff's position.
	TwoFloats position;
	///	The puff's direction.
	TwoFloats direction;
	///	The puff's rotation direction.
	float rotDir;
	///	The puff's speed.
	float speed;
	///	The puff's alpha.
	float alpha;
	///	The puff's alpha tweener.
	LogTweener<float> alphaTween;

	///	The vertices of the puff image.
	array<TwoFloats, 4> vertices;
	///	Used to rotate the puff image.
	float x0;
	///	Used to rotate the puff image.
	float x1;
	///	Used to rotate the puff image.
	float coeff;
};

#endif
