//	StateBase.h - Base class for the GamePlayStates, so we have a shared
//				  interface for Player to call.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef STATEBASE_H_
#define STATEBASE_H_

#include "c0xHeaders.h"

#include <string>

class GenericFileSounder;
struct TwoFloats;
class Player;

///	Base class for the GamePlayStates, so we have a shared interface for Player to call.
class StateBase
{
  public:
	///	Destructor.
	virtual ~StateBase() {};

	///	Returns true if there's a wall in the way of the passed-in direction.
	virtual bool getWall(const TwoFloats& pos, int dir) const = 0;
	///	Returns true if the capslock is down.
	virtual bool getCapsLock() const = 0;
	///	Returns true if the passed-in space is free of other Players.
	virtual bool getSpaceIsFree(const TwoFloats& pos,
								const TwoFloats& dir,
								int& recursion) = 0;
	///	Returns the sounder.
	virtual shared_ptr<GenericFileSounder>& getSounder() = 0;
	///	Adds a bullet to the game.
	virtual void addBullet(int dir, Player *player) = 0;
	///	Adds an explosiotn to the game.
	virtual void addExplosion(Player *player) = 0;
	///	Adds a scale up icon to the game.
	virtual void addScaleUpIcon(const std::wstring& im, const TwoFloats& pos) = 0;
	///	Called by an AI player when it's made its choice in the timed game.
	virtual void aiChoiceMade(Player *player) = 0;
};

#endif
