//	TitleHighlight.cpp - Used to highlight the current title option.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "TitleHighlight.h"
#include "RainbowPalette.h"
#include "Random.h"

//------------------------------------------------------------------------------
TitleHighlight::TitleHighlight(const TwoFloats& pos, RainbowPalette& palette):
position(pos),
colour(palette.getColour(Random::getGlobal().getRandomFloat())),
colTween(colour, palette.getColour(Random::getGlobal().getRandomFloat()), 0.01f),
rayRotate(Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI),
colourPalette(palette),
x0(0.0f),
x1(1.0f),
coeff(2.0f/60.0f)
{
	
}

//------------------------------------------------------------------------------
void TitleHighlight::update()
{
	if(!colTween.getFinished())
	{
		colTween.tick();
		if(colTween.getFinished())
			colTween.reset(colourPalette.getColour(Random::getGlobal().getRandomFloat()));
	}

	rayRotate += 0.01f;
	if(rayRotate > (2.0f * NIALL_PI))
		rayRotate -= 2.0f * NIALL_PI;

	x0 -= x1 * coeff;
	x1 += x0 * coeff;
}

//------------------------------------------------------------------------------
void TitleHighlight::drawRays(shared_ptr<Drawer>& d)
{
	for(int i=0;i<10;++i)
	{
		float rotateVal(rayRotate + (((float)i/10.0f) * 2.0f * NIALL_PI));

		d->drawRotImage(L"ExplodeRay",
						TwoFloats(position.x - 32.0f,
								  position.y - 192.0f),
						rotateVal,
						TwoFloats(32.0f, 192.0f),
						1.0f,
						colour + 0.33f);

		rotateVal = rayRotate + (((((float)i)+0.5f)/10.0f) * 2.0f * NIALL_PI);
		d->drawRotImage(L"ExplodeRay",
						TwoFloats(position.x - 32.0f,
								  position.y - 192.0f),
						rotateVal,
						TwoFloats(32.0f, 192.0f),
						1.0f,
						colour + 0.33f,
						0.5f);
	}

	float scale((x0 * 0.1f) + 0.9f);
	d->drawImage(L"Shockwave",
				 position - (281.0f * 0.5f * scale),
				 scale,
				 colour + 0.1f,
				 0.5f);
}

//------------------------------------------------------------------------------
void TitleHighlight::setPosition(const TwoFloats& pos)
{
	position = pos;
}
