//	TitleHighlight.h - Used to highlight the current title option.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TITLEHIGHLIGHT_H_
#define TITLEHIGHLIGHT_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

#include <vector>

class RainbowPalette;
class Drawer;
class Player;

///	Class representing an TitleHighlight.
class TitleHighlight
{
  public:
	///	Constructor.
	TitleHighlight(const TwoFloats& pos, RainbowPalette& palette);

	///	Updates the TitleHighlight.
	void update();
	///	Draws the TitleHighlight's rays.
	void drawRays(shared_ptr<Drawer>& d);

	///	Sets the highlight's position.
	void setPosition(const TwoFloats& pos);
  private:
	///	The TitleHighlight's position.
	TwoFloats position;
	///	The TitleHighlight's colour.
	ThreeFloats colour;
	///	Used to tween the TitleHighlight's colour.
	SmoothstepTweener<ThreeFloats> colTween;
	///	Our colour palette.
	RainbowPalette& colourPalette;

	///	Used to scale the shockwave ring.
	float x0;
	///	Used to scale the shockwave ring.
	float x1;
	///	Used to scale the shockwave ring.
	float coeff;

	///	Used to rotate the TitleHighlight's rays.
	float rayRotate;
};

#endif
