//	Wall.cpp - Class representing one of the enclosing walls.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "RainbowPalette.h"
#include "Wall.h"

//------------------------------------------------------------------------------
Wall::Wall(const TwoFloats& topLeft, RainbowPalette& palette, float index):
position(topLeft),
colourPalette(palette),
topIndex(index),
topTween(topColour, topColour, 0.05f),
bottomTween(bottomColour, bottomColour, 0.05f)
{
	bottomIndex = topIndex - 0.1f;
	if(bottomIndex < 0.0f)
		bottomIndex += 1.0f;

	topColour = colourPalette.getBackgroundColour(topIndex);
	bottomColour = colourPalette.getBackgroundColour(bottomIndex);

	topIndex += 0.05f;
	if(topIndex > 1.0f)
		topIndex -= 1.0f;
	topTween.reset(1.0f);

	bottomIndex += 0.05f;
	if(bottomIndex > 1.0f)
		bottomIndex -= 1.0f;
	bottomTween.reset(colourPalette.getBackgroundColour(bottomIndex));
}

//------------------------------------------------------------------------------
void Wall::update()
{
	if(!topTween.getFinished())
	{
		topTween.tick();
		if(topTween.getFinished())
		{
			/*topIndex += 0.05f;
			if(topIndex > 1.0f)
				topIndex -= 1.0f;
			topTween.reset(colourPalette.getBackgroundColour(topIndex));*/
		}
	}

	if(!bottomTween.getFinished())
	{
		bottomTween.tick();
		if(bottomTween.getFinished())
		{
			bottomIndex += 0.05f;
			if(bottomIndex > 1.0f)
				bottomIndex -= 1.0f;
			bottomTween.reset(colourPalette.getBackgroundColour(bottomIndex));
		}
	}
}

//------------------------------------------------------------------------------
void Wall::draw(shared_ptr<Drawer>& d)
{
	const TwoFloats& drawSize(d->getDrawingAreaSize());

	//Top.
	d->fillRect(position,
				TwoFloats(drawSize.x - (position.x * 2.0f), 80.0f),
				topColour,
				0.8f);

	//Bottom.
	d->fillRect(TwoFloats(position.x, (drawSize.y - position.y - 80.0f)),
				TwoFloats(drawSize.x - (position.x * 2.0f), 80.0f),
				topColour,
				0.8f);

	//Left.
	d->fillRect(TwoFloats(position.x, position.y + 80.0f),
				TwoFloats(80.0f, (drawSize.y - (position.y * 2.0f) - 160.0f)),
				topColour,
				0.8f);

	//Right.
	d->fillRect(TwoFloats((drawSize.x - position.x - 80.0f), position.y + 80.0f),
				TwoFloats(80.0f, (drawSize.y - (position.y * 2.0f) - 160.0f)),
				topColour,
				0.8f);
}
