//	Wall.h - Class representing one of the enclosing walls.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef WALL_H_
#define WALL_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

class RainbowPalette;
class Drawer;

///	Class representing one of the enclosing walls.
class Wall
{
  public:
	///	Constructor.
	Wall(const TwoFloats& topLeft, RainbowPalette& palette, float index);

	///	Updates the wall.
	void update();
	///	Draws the wall.
	void draw(shared_ptr<Drawer>& d);

	///	Returns this wall's current index w/in the colour palette.
	float getPaletteIndex() const {return topIndex;};
  private:
	///	The wall's top-left position.
	TwoFloats position;
	///	The wall's top colour.
	ThreeFloats topColour;
	///	wall's bottom colour.
	ThreeFloats bottomColour;
	///	The wall's colour palette.
	RainbowPalette& colourPalette;
	///	The index of the top colour.
	float topIndex;
	///	The index of the bottom colour.
	float bottomIndex;
	///	The top colour tweener.
	SmoothstepTweener<ThreeFloats> topTween;
	///	The bottom colour tweener.
	SmoothstepTweener<ThreeFloats> bottomTween;
};

#endif
