//	Wall2.cpp - Second class representing one of the enclosing walls.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "RainbowPalette.h"
#include "Random.h"
#include "Wall2.h"

//------------------------------------------------------------------------------
Wall2::Wall2(const TwoFloats& pos, RainbowPalette& palette):
position(pos - TwoFloats(0.0f, 48.0f)),
size(0.7f, 1.3f),
alpha(0.0f),
colTween(colour, colour, 0.01f),
colourPalette(palette),
posTween(position, pos, 0.05f),
sizeTween(size, 1.0f, 0.05f),
alphaTween(alpha, 1.0f, 0.75f)
{
	colour = colourPalette.getBackgroundColour(Random::getGlobal().getRandomFloat()) + 0.0f;
	colTween.reset(colourPalette.getBackgroundColour(Random::getGlobal().getRandomFloat()) + 0.0f);
}

//------------------------------------------------------------------------------
void Wall2::update()
{
	if(!colTween.getFinished())
	{
		colTween.tick();
		if(colTween.getFinished())
			colTween.reset(colourPalette.getBackgroundColour(Random::getGlobal().getRandomFloat()) + 0.0f);
	}

	if(!posTween.getFinished())
		posTween.tick();
	if(!sizeTween.getFinished())
		sizeTween.tick();
	if(!alphaTween.getFinished())
		alphaTween.tick();
}

//------------------------------------------------------------------------------
void Wall2::draw(shared_ptr<Drawer>& d)
{
	d->addImageToBatch(L"Wall",
					   position - (size * 37.0f),
					   size,
					   colour,
					   alpha);
}
