//	Wall2.h - Second class representing one of the enclosing walls.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef WALL2_H_
#define WALL2_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

class RainbowPalette;
class Drawer;

///	Second class representing one of the enclosing walls.
class Wall2
{
  public:
	///	Constructor.
	Wall2(const TwoFloats& pos, RainbowPalette& palette);

	///	Updates the wall.
	void update();
	///	Draws the wall.
	void draw(shared_ptr<Drawer>& d);

	///	Returns the wall's position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the wall's destination.
	const TwoFloats& getDestination() const {return posTween.getEndVal();};
  private:
	///	The wall's position.
	TwoFloats position;
	///	The wall's 2d size.
	TwoFloats size;
	///	The wall's colour.
	ThreeFloats colour;
	///	The wall's alpha.
	float alpha;
	///	Used to tween the wall's colour.
	SmoothstepTweener<ThreeFloats> colTween;
	///	The colour palette we use for the colour.
	RainbowPalette& colourPalette;

	///	Used to drop the wall in from above.
	Tweener<TwoFloats> posTween;
	///	Used to stretch and squash the wall vertically as it drops.
	Tweener<TwoFloats> sizeTween;
	///	Used to quickly fade the wall in.
	LogTweener<float> alphaTween;
};

#endif
