//	Stb_imageLoader.cpp - Class used to load images from different file
//						  formats.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/ImageLoader.h"
#include "DebugHeaders.h"
#include "UTF8File.h"
#include "stb_image.h"

//------------------------------------------------------------------------------
ImageLoader::ImageLoader()
{

}

//------------------------------------------------------------------------------
ImageLoader::~ImageLoader()
{

}

//------------------------------------------------------------------------------
ImageData ImageLoader::loadImage(const char *data, uint32_t size)
{
	ImageData retval;
	unsigned char *imData;
	int width;
	int height;
	int numComponents;

	imData = stbi_load_from_memory((const unsigned char *)data,
								   size,
								   &width,
								   &height,
								   &numComponents,
								   4);
	if(!imData)
	{
		DebugStatement(L"Stb_imageLoader: Could not load image.");

		retval.data = 0;
		retval.width = 0;
		retval.height = 0;
		return retval;
	}

	retval.data = NiallNew uint8_t[width*height*4];
	for(int i=0;i<(width*height*4);++i)
			retval.data[i] = imData[i];
	retval.width = width;
	retval.height = height;

	//Premultiply the alpha.
	for(int i=0;i<(width*height);++i)
	{
		float tempf;
		float alpha = (float)retval.data[(i*4)+3]/255.0f;

		tempf = (float)retval.data[(i*4)]/255.0f;
		retval.data[(i*4)] = (uint8_t)(tempf * alpha * 255.0f);
		tempf = (float)retval.data[(i*4)+1]/255.0f;
		retval.data[(i*4)+1] = (uint8_t)(tempf * alpha * 255.0f);
		tempf = (float)retval.data[(i*4)+2]/255.0f;
		retval.data[(i*4)+2] = (uint8_t)(tempf * alpha * 255.0f);
	}

	free(imData);

	return retval;
}

//------------------------------------------------------------------------------
ImageData ImageLoader::loadImage(const FilePath& path)
{
	ImageData retval;
	unsigned char *imData;
	int width;
	int height;
	int numComponents;
	std::string narrowPath;

	UTF8File::wstringToChar(path.getPath(), narrowPath);
	imData = stbi_load(narrowPath.c_str(),
					   &width,
					   &height,
					   &numComponents,
					   4);
	if(!imData)
	{
		DebugStatement(L"Stb_imageLoader: Could not load image from path: " << path.getPath());

		retval.data = 0;
		retval.width = 0;
		retval.height = 0;
		return retval;
	}

	retval.data = NiallNew uint8_t[width*height*4];
	for(int i=0;i<(width*height*4);++i)
		retval.data[i] = imData[i];
	retval.width = width;
	retval.height = height;

	//Premultiply the alpha.
	for(int i=0;i<(width*height);++i)
	{
		float tempf;
		float alpha = (float)retval.data[(i*4)+3]/255.0f;

		tempf = (float)retval.data[(i*4)]/255.0f;
		retval.data[(i*4)] = (uint8_t)(tempf * alpha * 255.0f);
		tempf = (float)retval.data[(i*4)+1]/255.0f;
		retval.data[(i*4)+1] = (uint8_t)(tempf * alpha * 255.0f);
		tempf = (float)retval.data[(i*4)+2]/255.0f;
		retval.data[(i*4)+2] = (uint8_t)(tempf * alpha * 255.0f);
	}

	free(imData);

	return retval;
}
