//	ScalesAndFrequencies.cpp - Helper stuff for working with scales.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ScalesAndFrequencies.h"
#include "Random.h"

#include <algorithm>

//------------------------------------------------------------------------------
ScalesAndFrequencies::ScalesAndFrequencies()
{
	int i(0);
	//These scales taken from a tweet by Stephen Lavelle.
	int tempScales[][12] = {{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
							{1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0},
							{1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0},
							{1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0},
							{1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1},
							{1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0},
							{1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
							{1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0},
							{1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0},
							{1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 0},
							{1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0},
							{1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0},
							{1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0},
							{1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0},
							{1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1},
							{1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0},
							{1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0},
							{1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
							{1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1},
							{1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1},
							{1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0},
							{1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 0},
							{1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1},
							{1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1},
							{1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1},
							{1, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0},
							{1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0},
							{1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1},
							{1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0},
							{1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1},
							{1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0},
							{1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0},
							{1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1},
							{1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0},
							{1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0},
							{1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1, 0},
							{1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 0, 1},
							{1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 0},
							{1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1},
							{1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1},
							{1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0},
							{1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0},
							{1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1},
							{1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0},
							{1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0},
							{1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1},
							{1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1},
							{1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1},
							{1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0},
							{1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0},
							{1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0},
							{1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0},
							{1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0},
							{1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0},
							{1, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
							{1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0},
							{1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0},
							{1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0},
							{1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0},
							{1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1}};

	addScale(L"Chromatic", tempScales[i++]);
	addScale(L"Spanish", tempScales[i++]);
	addScale(L"Flamenco", tempScales[i++]);
	addScale(L"Symmetrical", tempScales[i++]);
	addScale(L"Diminished", tempScales[i++]);
	addScale(L"Wholetone", tempScales[i++]);
	addScale(L"Augmented", tempScales[i++]);
	addScale(L"3Semitone", tempScales[i++]);
	addScale(L"4Semitone", tempScales[i++]);
	addScale(L"UltraLocrian", tempScales[i++]);
	addScale(L"SuperLocrian", tempScales[i++]);
	addScale(L"Indian", tempScales[i++]);
	addScale(L"Locrian", tempScales[i++]);
	addScale(L"Phrygian", tempScales[i++]);
	addScale(L"NeapolitanMajor", tempScales[i++]);
	addScale(L"Blues", tempScales[i++]);
	addScale(L"Javanese", tempScales[i++]);
	addScale(L"NeapolitanMinor", tempScales[i++]);
	addScale(L"Todi", tempScales[i++]);
	addScale(L"Persian", tempScales[i++]);
	addScale(L"Oriental", tempScales[i++]);
	addScale(L"PhrygianMajor", tempScales[i++]);
	addScale(L"DoubleHarmonic", tempScales[i++]);
	addScale(L"Marva", tempScales[i++]);
	addScale(L"Enigmatic", tempScales[i++]);
	addScale(L"LocrianNatural2nd", tempScales[i++]);
	addScale(L"NaturalMinor", tempScales[i++]);
	addScale(L"HarmonicMinor", tempScales[i++]);
	addScale(L"Dorian", tempScales[i++]);
	addScale(L"MelodicMinorAsc", tempScales[i++]);
	addScale(L"MelodicMinorDesc", tempScales[i++]);
	addScale(L"HungarianGypsy", tempScales[i++]);
	addScale(L"HungarianMinor", tempScales[i++]);
	addScale(L"Romanian", tempScales[i++]);
	addScale(L"LocrianMajor", tempScales[i++]);
	addScale(L"Hindu", tempScales[i++]);
	addScale(L"Ethiopian", tempScales[i++]);
	addScale(L"Mixolydian", tempScales[i++]);
	addScale(L"Major", tempScales[i++]);
	addScale(L"MajorBebop", tempScales[i++]);
	addScale(L"MinorBebop", tempScales[i++]);
	addScale(L"MixolydianAugmented", tempScales[i++]);
	addScale(L"HarmonicMajor", tempScales[i++]);
	addScale(L"LydianMinor", tempScales[i++]);
	addScale(L"LydianDominant", tempScales[i++]);
	addScale(L"Lydian", tempScales[i++]);
	addScale(L"LydianAugmented", tempScales[i++]);
	addScale(L"LydianWholetone", tempScales[i++]);
	addScale(L"RockNRoll", tempScales[i++]);
	addScale(L"HungarianMajor", tempScales[i++]);
	addScale(L"Balinese", tempScales[i++]);
	addScale(L"Pelog", tempScales[i++]);
	addScale(L"Iwato", tempScales[i++]);
	addScale(L"Japanese", tempScales[i++]);
	addScale(L"Insen", tempScales[i++]);
	addScale(L"Hirajoshi", tempScales[i++]);
	addScale(L"PentatonicMajor", tempScales[i++]);
	addScale(L"Egyptian", tempScales[i++]);
	addScale(L"PentatonicMinor", tempScales[i++]);
	addScale(L"Chinese", tempScales[i++]);
}

//------------------------------------------------------------------------------
void ScalesAndFrequencies::getFrequencies(std::vector<float>& frequencies,
										  int root,
										  int octaves,
										  std::wstring& scale)
{
	std::map<std::wstring, std::vector<int> >::iterator it(scales.find(scale));

	if((scale.length() == 0) || (it == scales.end()))
		scale = randomScales[Random::getGlobal().getRandomInt(randomScales.size())];

	for(int i=root, j=0;i<(root + (octaves*12) + 1);)
	{
		frequencies.push_back(midiFreqs[i]);

		i += scales[scale][j];
		++j;
		if(j >= (int)scales[scale].size())
			j = 0;
	}
}

//------------------------------------------------------------------------------
void ScalesAndFrequencies::chooseScales(const std::vector<std::wstring>& scaleSet)
{
	randomScales.clear();

	for(unsigned int i=0;i<scaleSet.size();++i)
		randomScales.push_back(scaleSet[i]);
}

//------------------------------------------------------------------------------
void ScalesAndFrequencies::removeScale(const std::wstring& scale)
{
	std::vector<std::wstring>::iterator it;

	it = std::find(randomScales.begin(), randomScales.end(), scale);

	if(it != randomScales.end())
		randomScales.erase(it);
}

//------------------------------------------------------------------------------
void ScalesAndFrequencies::addScale(const std::wstring& scale,
									const int notes[12])
{
	std::vector<int> tempScale;

	scales.insert(std::make_pair(scale, std::vector<int>()));
	randomScales.push_back(scale);

	//We count backwards from the end.
	for(int i=11,j=1;i>=0;)
	{
		while(notes[i] < 1)
		{
			--i;
			++j;
		}
		tempScale.push_back(j);
		--i;
		j = 1;
	}
	//Now put the scale into scales[scale] in the correct order.
	for(int i=(tempScale.size()-1);i>=0;--i)
		scales[scale].push_back(tempScale[i]);

	/*for(int i=0,j=0;i<12;)
	{
		scales[scale].push_back(j);

		++i;
		j = 1;
		if(i < 12)
		{
			while((notes[i] < 1) && (i < 12))
			{
				++i;
				++j;
			}
		}
	}*/
}

//------------------------------------------------------------------------------
float ScalesAndFrequencies::midiFreqs[128] = {
	8.1757989156f, //0
	8.6619572180f,
	9.1770239974f,
	9.7227182413f,
	10.3008611535f,
	10.9133822323f,
	11.5623257097f,
	12.2498573744f,
	12.9782717994f,
	13.7500000000f,
	14.5676175474f,
	15.4338531643f,
	16.3515978313f, //12
	17.3239144361f,
	18.3540479948f,
	19.4454364826f,
	20.6017223071f,
	21.8267644646f,
	23.1246514195f,
	24.4997147489f,
	25.9565435987f,
	27.5000000000f,
	29.1352350949f,
	30.8677063285f,
	32.7031956626f, //24
	34.6478288721f,
	36.7080959897f,
	38.8908729653f,
	41.2034446141f,
	43.6535289291f,
	46.2493028390f,
	48.9994294977f,
	51.9130871975f,
	55.0000000000f,
	58.2704701898f,
	61.7354126570f,
	65.4063913251f, //36
	69.2956577442f,
	73.4161919794f,
	77.7817459305f,
	82.4068892282f,
	87.3070578583f,
	92.4986056779f,
	97.9988589954f,
	103.8261743950f,
	110.0000000000f,
	116.5409403795f,
	123.4708253140f,
	130.8127826503f, //48
	138.5913154884f,
	146.8323839587f,
	155.5634918610f,
	164.8137784564f,
	174.6141157165f,
	184.9972113558f,
	195.9977179909f,
	207.6523487900f,
	220.0000000000f,
	233.0818807590f,
	246.9416506281f,
	261.6255653006f, //60
	277.1826309769f,
	293.6647679174f,
	311.1269837221f,
	329.6275569129f,
	349.2282314330f,
	369.9944227116f,
	391.9954359817f,
	415.3046975799f,
	440.0000000000f,
	466.1637615181f,
	493.8833012561f,
	523.2511306012f, //72
	554.3652619537f,
	587.3295358348f,
	622.2539674442f,
	659.2551138257f,
	698.4564628660f,
	739.9888454233f,
	783.9908719635f,
	830.6093951599f,
	880.0000000000f,
	932.3275230362f,
	987.7666025122f,
	1046.5022612024f, //84
	1108.7305239075f,
	1174.6590716696f,
	1244.5079348883f,
	1318.5102276515f,
	1396.9129257320f,
	1479.9776908465f,
	1567.9817439270f,
	1661.2187903198f,
	1760.0000000000f,
	1864.6550460724f,
	1975.5332050245f,
	2093.0045224048f, //96
	2217.4610478150f,
	2349.3181433393f,
	2489.0158697766f,
	2637.0204553030f,
	2793.8258514640f,
	2959.9553816931f,
	3135.9634878540f,
	3322.4375806396f,
	3520.0000000000f,
	3729.3100921447f,
	3951.0664100490f,
	4186.009044809f, //108
	4434.922095630f,
	4698.636286678f,
	4978.031739553f,
	5274.040910605f,
	5587.651702928f,
	5919.910763386f,
	6271.926975708f,
	6644.875161279f,
	7040.000000000f,
	7458.620234756f,
	7902.132834658f,
	8372.0180896192f, //120
	8869.8441912599f,
	9397.2725733570f,
	9956.0634791066f,
	10548.0818212118f,
	11175.3034058561f,
	11839.8215267723f,
	12543.8539514160f };
